"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getHighlighter = getHighlighter;
exports.getSafeLanguage = getSafeLanguage;
exports.getThemeContext = getThemeContext;
exports.isValidLanguage = isValidLanguage;
var _core = require("@shikijs/core");
var _engineJavascript = require("@shikijs/engine-javascript");
var _vscodeTextmate = require("@shikijs/vscode-textmate");
var _css = _interopRequireDefault(require("shiki/dist/langs/css.mjs"));
var _html = _interopRequireDefault(require("shiki/dist/langs/html.mjs"));
var _javascript = _interopRequireDefault(require("shiki/dist/langs/javascript.mjs"));
var _json = _interopRequireDefault(require("shiki/dist/langs/json.mjs"));
var _markdown = _interopRequireDefault(require("shiki/dist/langs/markdown.mjs"));
var _php = _interopRequireDefault(require("shiki/dist/langs/php.mjs"));
var _tsx = _interopRequireDefault(require("shiki/dist/langs/tsx.mjs"));
var _typescript = _interopRequireDefault(require("shiki/dist/langs/typescript.mjs"));
var _githubDark = _interopRequireDefault(require("shiki/dist/themes/github-dark.mjs"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/* eslint-disable no-console */

const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args);
  }
}
// Theme normalization helper
function normalizeTheme(rawTheme) {
  const theme = {
    ...rawTheme
  };

  // Set defaults
  theme.type ||= 'dark';
  theme.settings ||= [];
  theme.colorReplacements ||= {};

  // Fallback settings
  if (theme.tokenColors && !theme.settings) {
    theme.settings = theme.tokenColors;
    delete theme.tokenColors;
  }
  theme.settings ||= [];
  theme.bg ||= theme.type === 'dark' ? '#1e1e1e' : '#ffffff';
  theme.fg ||= theme.type === 'dark' ? '#bbbbbb' : '#333333';

  // Push a no-scope setting with fallback colors
  if (!theme.settings[0] || !theme.settings[0].settings || theme.settings[0].scope) {
    theme.settings.unshift({
      settings: {
        foreground: theme.fg,
        background: theme.bg
      }
    });
  }
  return theme;
}

// Process language grammars
function processLanguage(lang) {
  if (!lang) {
    log('Warning: Null language definition');
    return null;
  }
  const grammar = lang.grammar || {};
  const repository = grammar.repository || {};
  const patterns = grammar.patterns || [];

  // Add core metadata
  return {
    ...lang,
    name: lang.name || lang.id,
    scopeName: lang.scopeName || `source.${lang.id || lang.name}`,
    embeddedLanguages: lang.embeddedLanguages || {},
    grammar: {
      ...grammar,
      repository,
      patterns
    }
  };
}

// Language registration map
const languageMap = {
  typescript: processLanguage(_typescript.default),
  javascript: processLanguage(_javascript.default),
  json: processLanguage(_json.default),
  html: processLanguage(_html.default),
  css: processLanguage(_css.default),
  markdown: processLanguage(_markdown.default),
  tsx: processLanguage(_tsx.default),
  php: processLanguage(_php.default),
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{
        match: '.*',
        name: 'text.plain'
      }]
    }
  }
};
let highlighterInstance = null;
const themeCache = new Map();
function getThemeContext(theme) {
  const themeId = typeof theme === 'string' ? theme : theme.name || 'default';
  if (!themeCache.has(themeId)) {
    const normalizedTheme = normalizeTheme(theme);
    const textmateTheme = _vscodeTextmate.Theme.createFromRawTheme(normalizedTheme);
    themeCache.set(themeId, {
      theme: normalizedTheme,
      textmateTheme
    });
  }
  return themeCache.get(themeId);
}
async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance');
    return highlighterInstance;
  }
  try {
    log('Creating new highlighter instance...');

    // Process default theme
    const defaultTheme = getThemeContext(_githubDark.default);

    // Initialize regex engine
    const engine = (0, _engineJavascript.createJavaScriptRegexEngine)();

    // Process and validate language definitions
    log('Processing languages...');
    const validLanguages = Object.entries(languageMap).filter(([id, lang]) => {
      const isValid = lang && lang.scopeName && lang.grammar;
      if (!isValid) {
        log(`Warning: Invalid language registration for ${id}`);
      }
      return isValid;
    }).map(([_, lang]) => lang);
    log(`Validated languages: ${validLanguages.length}`);

    // Create highlighter with processed components
    highlighterInstance = await (0, _core.createHighlighterCore)({
      themes: [defaultTheme.theme],
      langs: validLanguages,
      engine
    });
    const loadedLangs = highlighterInstance.getLoadedLanguages();
    log('Successfully loaded languages:', loadedLangs);
    return highlighterInstance;
  } catch (error) {
    log('Error creating highlighter:', error);
    throw error;
  }
}
function isValidLanguage(lang) {
  return lang in languageMap;
}
function getSafeLanguage(lang) {
  const normalized = lang.toLowerCase();
  if (isValidLanguage(normalized)) {
    return normalized;
  }
  log(`Warning: Unsupported language '${lang}', falling back to plaintext`);
  return 'plaintext';
}
//# sourceMappingURL=SyntaxContext.js.map