"use strict";

/* eslint-disable no-console */

import React from 'react';
import { ActivityIndicator, ScrollView, Text, View } from 'react-native';
import { getHighlighter } from './SyntaxContext';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[SyntaxHighlighter] ${message}`, ...args);
  }
}
const defaultHighlightLines = [];
export const SyntaxHighlighter = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  highlightLines = defaultHighlightLines
}) => {
  const [loading, setLoading] = React.useState(true);
  const [error, setError] = React.useState(null);
  const [tokens, setTokens] = React.useState([]);
  React.useEffect(() => {
    const highlight = async () => {
      try {
        log('Starting highlighting process');
        log('Input:', {
          text: `${text.substring(0, 100)}...`,
          // Truncate for logging
          language,
          theme: theme.name,
          themeType: typeof theme
        });
        setLoading(true);
        const highlighter = await getHighlighter();
        log('Highlighter instance obtained');

        // Debug highlighter state
        log('Highlighter capabilities:', {
          hasLanguage: Boolean(highlighter.getLoadedLanguages().includes(language)),
          loadedLanguages: highlighter.getLoadedLanguages()
        });
        if (!text || !language || !theme) {
          throw new Error('Missing required props');
        }

        // Verify language is loaded
        log('Checking language support...');
        const loadedLangs = highlighter.getLoadedLanguages();
        log('Loaded languages:', loadedLangs);
        if (!loadedLangs.includes(language)) {
          log('WARNING: Language not found in loaded languages!');
        }

        // Debug theme
        log('Theme details:', {
          name: theme.name,
          settings: theme.settings?.length || 'none',
          tokenColors: theme.tokenColors?.length || 'none'
        });
        log('Attempting tokenization...');
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme
        });
        log('Tokenization completed');
        log('Result structure:', {
          hasTokens: Boolean(result?.tokens),
          tokenCount: result?.tokens?.length || 0
        });
        if (result?.tokens) {
          log('Sample tokens:', result.tokens[0]?.slice(0, 2));
        }
        if (!result?.tokens) {
          throw new Error('Invalid tokenization result');
        }
        setTokens(result.tokens);
        setError(null);
      } catch (err) {
        const errorMessage = err instanceof Error ? err.message : 'Failed to highlight code';
        log('Error during highlighting:', err);
        if (err instanceof Error) {
          log('Error stack:', err.stack);
        }
        log('State at error:', {
          textLength: text?.length,
          language,
          themeAvailable: Boolean(theme)
        });
        setError(errorMessage);
        setTokens([]);
      } finally {
        setLoading(false);
      }
    };
    highlight();
  }, [text, language, theme]);
  const renderToken = (token, index) => {
    return /*#__PURE__*/_jsx(Text, {
      style: [{
        color: token.color,
        fontSize,
        fontFamily: 'monospace',
        fontStyle: token.fontStyle === 2 ? 'italic' : 'normal',
        fontWeight: token.fontStyle === 1 ? 'bold' : 'normal'
      }, textStyle],
      children: token.content
    }, `${token.offset || index}-${index}`);
  };
  const renderLine = (lineTokens, index) => {
    const isHighlighted = highlightLines.includes(index + 1);
    return /*#__PURE__*/_jsx(View, {
      style: [{
        flexDirection: 'row',
        flexWrap: 'wrap',
        backgroundColor: isHighlighted ? 'rgba(255,255,255,0.1)' : 'transparent'
      }],
      children: lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))
    }, index);
  };
  if (loading) {
    return /*#__PURE__*/_jsx(View, {
      style: [{
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsx(ActivityIndicator, {})
    });
  }
  if (error) {
    return /*#__PURE__*/_jsx(View, {
      style: [{
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsxs(Text, {
        style: {
          color: '#ff6b6b'
        },
        children: ["Error:", error]
      })
    });
  }
  return /*#__PURE__*/_jsx(ScrollView, {
    ...scrollViewProps,
    horizontal: true,
    style: [{
      backgroundColor: theme.colors?.background || '#1e1e1e'
    }, scrollViewProps?.style],
    contentContainerStyle: [{
      padding: 10
    }, containerStyle],
    children: /*#__PURE__*/_jsx(View, {
      onStartShouldSetResponder: () => true,
      children: tokens.map((lineTokens, index) => renderLine(lineTokens, index))
    })
  });
};
//# sourceMappingURL=SyntaxHighlighter.js.map