import type { ThemeRegistrationAny, ThemeRegistrationResolved } from '@shikijs/types'
/* eslint-disable no-console */
import { createHighlighterCore, type HighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import { Theme as TextMateTheme } from '@shikijs/vscode-textmate'

import css from 'shiki/dist/langs/css.mjs'
import html from 'shiki/dist/langs/html.mjs'
import javascript from 'shiki/dist/langs/javascript.mjs'
import json from 'shiki/dist/langs/json.mjs'
import markdown from 'shiki/dist/langs/markdown.mjs'
import php from 'shiki/dist/langs/php.mjs'
import tsx from 'shiki/dist/langs/tsx.mjs'
import typescript from 'shiki/dist/langs/typescript.mjs'
import githubDark from 'shiki/dist/themes/github-dark.mjs'

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args)
  }
}

export type AvailableLanguage = 'php' | 'tsx' | 'typescript' | 'javascript' | 'json' | 'html' | 'css' | 'markdown' | 'plaintext'

// Theme normalization helper
function normalizeTheme(rawTheme: ThemeRegistrationAny): ThemeRegistrationResolved {
  const theme = {
    ...rawTheme,
  } as ThemeRegistrationResolved

  // Set defaults
  theme.type ||= 'dark'
  theme.settings ||= []
  theme.colorReplacements ||= {}

  // Fallback settings
  if (theme.tokenColors && !theme.settings) {
    theme.settings = theme.tokenColors
    delete theme.tokenColors
  }

  theme.settings ||= []
  theme.bg ||= theme.type === 'dark' ? '#1e1e1e' : '#ffffff'
  theme.fg ||= theme.type === 'dark' ? '#bbbbbb' : '#333333'

  // Push a no-scope setting with fallback colors
  if (!theme.settings[0] || !theme.settings[0].settings || theme.settings[0].scope) {
    theme.settings.unshift({
      settings: {
        foreground: theme.fg,
        background: theme.bg,
      },
    })
  }

  return theme
}

// Process language grammars
function processLanguage(lang: any): any {
  if (!lang) {
    log('Warning: Null language definition')
    return null
  }

  const grammar = lang.grammar || {}
  const repository = grammar.repository || {}
  const patterns = grammar.patterns || []

  // Add core metadata
  return {
    ...lang,
    name: lang.name || lang.id,
    scopeName: lang.scopeName || `source.${lang.id || lang.name}`,
    embeddedLanguages: lang.embeddedLanguages || {},
    grammar: {
      ...grammar,
      repository,
      patterns,
    },
  }
}

// Language registration map
const languageMap: Record<AvailableLanguage, any> = {
  typescript: processLanguage(typescript),
  javascript: processLanguage(javascript),
  json: processLanguage(json),
  html: processLanguage(html),
  css: processLanguage(css),
  markdown: processLanguage(markdown),
  tsx: processLanguage(tsx),
  php: processLanguage(php),
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{ match: '.*', name: 'text.plain' }],
    },
  },
}

let highlighterInstance: HighlighterCore | null = null

interface ThemeContext {
  theme: ThemeRegistrationResolved
  textmateTheme: TextMateTheme
}

const themeCache = new Map<string, ThemeContext>()

export function getThemeContext(theme: ThemeRegistrationAny): ThemeContext {
  const themeId = typeof theme === 'string' ? theme : theme.name || 'default'

  if (!themeCache.has(themeId)) {
    const normalizedTheme = normalizeTheme(theme)
    const textmateTheme = TextMateTheme.createFromRawTheme(normalizedTheme)
    themeCache.set(themeId, {
      theme: normalizedTheme,
      textmateTheme,
    })
  }

  return themeCache.get(themeId)!
}

export async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance')
    return highlighterInstance
  }

  try {
    log('Creating new highlighter instance...')

    // Process default theme
    const defaultTheme = getThemeContext(githubDark)

    // Initialize regex engine
    const engine = createJavaScriptRegexEngine()

    // Process and validate language definitions
    log('Processing languages...')
    const validLanguages = Object.entries(languageMap)
      .filter(([id, lang]) => {
        const isValid = lang && lang.scopeName && lang.grammar
        if (!isValid) {
          log(`Warning: Invalid language registration for ${id}`)
        }
        return isValid
      })
      .map(([_, lang]) => lang)

    log(`Validated languages: ${validLanguages.length}`)

    // Create highlighter with processed components
    highlighterInstance = await createHighlighterCore({
      themes: [defaultTheme.theme],
      langs: validLanguages,
      engine,
    })

    const loadedLangs = highlighterInstance.getLoadedLanguages()
    log('Successfully loaded languages:', loadedLangs)

    return highlighterInstance
  }
  catch (error) {
    log('Error creating highlighter:', error)
    throw error
  }
}

export function isValidLanguage(lang: string): lang is AvailableLanguage {
  return lang in languageMap
}

export function getSafeLanguage(lang: string): AvailableLanguage {
  const normalized = lang.toLowerCase()
  if (isValidLanguage(normalized)) {
    return normalized as AvailableLanguage
  }
  log(`Warning: Unsupported language '${lang}', falling back to plaintext`)
  return 'plaintext'
}
